//application.cpp
//ver 1.02    
//- MyVector2 Ŭ ߰

#include <windowsx.h>
#include "graphics.h"
#include "application.h"

CApplication * CApplication::m_hApplication = NULL;
extern LRESULT WINAPI WindowProc(HWND hWnd, unsigned int msg, WPARAM wParam, LPARAM lParam);

//------------------------------------------------------------------------------------------------------
LRESULT WINAPI CApplication::WindowProc(HWND hWnd, unsigned int msg, WPARAM wParam, LPARAM lParam)
{
	return CApplication::GetInstance()->ProcessMessage(hWnd, msg, wParam, lParam);
}
//------------------------------------------------------------------------------------------------------
CApplication::CApplication() 
: m_pGraphics(NULL), m_bActive(false)
{
	m_mainParam.hInstance = 0;
	m_mainParam.lpCmdLine = NULL;
	m_mainParam.nCmdShow = 0;

	m_windowRef.hWnd = NULL;
	m_windowRef.iWidth = 0;
	m_windowRef.iHeight = 0;
	m_windowRef.bIsFullScreen = false;

	m_cameraRef.bRotateCamera = false;
	m_cameraRef.wheel = 0.0f;
	m_cameraRef.fRotateX = 0.0f;
	m_cameraRef.fRotateY = 0.0f;
}
//------------------------------------------------------------------------------------------------------
CApplication::~CApplication()
{
}
//------------------------------------------------------------------------------------------------------
bool CApplication::Run()
{
	if(CreateMainWindow() == false)
		return false;

	if(CreateGraphics() == false)
		return false;

	MessageLoop();
	return true;
}
//------------------------------------------------------------------------------------------------------
void CApplication::MessageLoop()
{
	MSG msg;

	BOOL bGotMsg;

	msg.message = WM_NULL;
	PeekMessage( &msg, NULL, 0U, 0U, PM_NOREMOVE );
    while( WM_QUIT != msg.message  )
    {
        // Use PeekMessage() if the app is active, so we can use idle time to
        // render the scene. Else, use GetMessage() to avoid eating CPU time.
        if( m_bActive )
            bGotMsg = PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE );
        else
            bGotMsg = GetMessage( &msg, NULL, 0U, 0U );

        if( bGotMsg )
        {
            // Translate and dispatch the message
            //if( 0 == TranslateAccelerator( g_hWnd, hAccel, &msg ) )
            {
                TranslateMessage( &msg );
                DispatchMessage( &msg );
            }
        }
        else
        {
            // Render a frame during idle time (no messages are waiting)
            if( m_bActive && m_pGraphics)
            {
				HRESULT hr = m_pGraphics->RestoreDevice();
				if( FAILED( hr ) && hr != D3DERR_DEVICELOST)
					SendMessage( m_windowRef.hWnd, WM_CLOSE, 0, 0 );

				if( FAILED(RenderFrame())  )
					SendMessage( m_windowRef.hWnd, WM_CLOSE, 0, 0 );
            }
        }
    }

	UnregisterClass( m_windowRef.strAppName.c_str(), m_mainParam.hInstance);
}
//---------------------------------------------------------------------------------------------------------
void CApplication::MoveMouse(int x, int y)
{
    if( m_cameraRef.bRotateCamera )
    {
        m_cameraRef.fRotateX -= (x - m_lastMousePos.x);
        m_cameraRef.fRotateY -= (y - m_lastMousePos.y);
    }

	m_lastMousePos.x = x;
	m_lastMousePos.y = y;
}
//------------------------------------------------------------------------------------------------------
LRESULT WINAPI CApplication::ProcessMessage(HWND hWnd, unsigned int msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
	case WM_ACTIVATE:
		m_bActive = (LOWORD(wParam) == 0) ? false : true;
		return 0;
		break;

    case WM_LBUTTONDOWN:
        m_lastMousePos.x = GET_X_LPARAM (lParam);   m_lastMousePos.y = GET_Y_LPARAM (lParam);
		m_cameraRef.bRotateCamera = true;
        break;
    case WM_LBUTTONUP:
        m_cameraRef.bRotateCamera = false;
		break;
	case WM_MOUSEWHEEL:
		{
			int zDelta = GET_WHEEL_DELTA_WPARAM(wParam);
			m_cameraRef.wheel += ((float)zDelta / 1200.0f);
		}
		break;
    case WM_MOUSEMOVE:
		MoveMouse(GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
		break;

	case WM_KEYDOWN:
		{
			if(ProcessKey(hWnd, wParam))
				return 0;
		}
		break;

	case WM_DESTROY:
		Cleanup();
		PostQuitMessage(0);
		return 0;

	case WM_SYSCOMMAND:
		// Prevent moving/sizing and power loss in fullscreen mode
		switch( wParam )
		{
			case SC_MOVE:
			case SC_SIZE:
			case SC_MAXIMIZE:
			case SC_KEYMENU:
			case SC_MONITORPOWER:
				if( m_windowRef.bIsFullScreen)
					return 1;
				break;
		}
		break;

    case WM_NCHITTEST:
       // Prevent the user from selecting the menu in fullscreen mode
        if(m_windowRef.bIsFullScreen)
              return HTCLIENT;
        break;
/*  콺 Ŀ ʿ   Ѵ
	case WM_SETCURSOR:
		SetCursor (NULL);
		return 0;
		break; 
*/
	};

	return DefWindowProc(hWnd, msg, wParam, lParam);
}
//------------------------------------------------------------------------------------------------------
bool CApplication::ProcessKey(HWND hWnd, WPARAM wKey)
{
	if (wKey == VK_ESCAPE)
	{
		DestroyWindow(hWnd);
		true;
	}
	return false;
}
//------------------------------------------------------------------------------------------------------
void CApplication::Cleanup()
{
	SAFE_DELETE(m_pGraphics);
}
//------------------------------------------------------------------------------------------------------
bool CApplication::CreateMainWindow()
{
	PrepareWindow();
	InitWindow();
	PostWindow();
	return true;
}
//------------------------------------------------------------------------------------------------------
bool CApplication::PrepareWindow()
{
	return true;
}
//------------------------------------------------------------------------------------------------------
bool CApplication::InitWindow()
{
	WNDCLASS		    wc;
		
	memset(&wc, 0, sizeof(WNDCLASS));
	wc.hInstance		= m_mainParam.hInstance;
	wc.lpszClassName	= m_windowRef.strAppName.c_str();
	wc.lpfnWndProc		= CApplication::WindowProc;

	wc.hIcon			= LoadIcon (NULL, IDC_ICON);
	wc.hCursor			= LoadCursor (NULL, IDC_ARROW);
	// Ϲ ø̸̼ 귯  ش.
	//wc.hbrBackground	= (HBRUSH) GetStockObject (BLACK_BRUSH);

	RegisterClass(&wc);

	if(m_windowRef.bIsFullScreen)
	{
		m_windowRef.hWnd = CreateWindowEx (
			NULL,
			m_windowRef.strAppName.c_str(),
			m_windowRef.strAppName.c_str(),
			WS_POPUP | WS_SYSMENU | WS_VISIBLE, 
			0, 0, 
			m_windowRef.iWidth,
			m_windowRef.iHeight,
			NULL,
			NULL,
			m_mainParam.hInstance,
			NULL);
	}
	else
	{
		int sx = GetSystemMetrics (SM_CXSCREEN);
		int sy = GetSystemMetrics (SM_CYSCREEN);

		m_windowRef.hWnd = CreateWindowEx (
			NULL,
			m_windowRef.strAppName.c_str(),
			m_windowRef.strAppName.c_str(),
			WS_OVERLAPPED | WS_SYSMENU,
			sx/2 - m_windowRef.iWidth/2, sy/2 - m_windowRef.iHeight/2,
			m_windowRef.iWidth,
			m_windowRef.iHeight,
			NULL,
			NULL,
			m_mainParam.hInstance,
			NULL);
	};

	if(!m_windowRef.hWnd) 
		return false;

	ShowWindow (m_windowRef.hWnd, m_mainParam.nCmdShow);
	UpdateWindow (m_windowRef.hWnd);	
	return true;

}
//------------------------------------------------------------------------------------------------------
bool CApplication::PostWindow()
{
	return true;
}
//------------------------------------------------------------------------------------------------------
bool CApplication::CreateGraphics()
{
	PrepareGraphics();
	InitGraphics();
	PostGraphics();
	return true;
}
//------------------------------------------------------------------------------------------------------
bool CApplication::PrepareGraphics()
{
	return true;
}
//------------------------------------------------------------------------------------------------------
bool CApplication::InitGraphics()
{
	m_pGraphics = new CGraphics();
	m_pGraphics->Create(m_windowRef.hWnd, m_windowRef.iWidth, m_windowRef.iHeight, m_windowRef.bIsFullScreen);
	m_pGraphics->SetupCamera();
	return true;
}
//------------------------------------------------------------------------------------------------------
bool CApplication::PostGraphics()
{
	if(m_pGraphics == NULL)
		return true;

	IDirect3DDevice9* pDevice = m_pGraphics->GetDevice();
	if(pDevice)
	{
		pDevice->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
		pDevice->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
		pDevice->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_POINT);
		pDevice->SetRenderState(D3DRS_LIGHTING, FALSE);	

		pDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CW);
	}
	return true;
}
//------------------------------------------------------------------------------------------------------
HRESULT CApplication::RenderFrame()
{
	if(m_pGraphics == NULL)
		return S_OK;

	Update();
	m_pGraphics->BeginScene();
	Render();
	return m_pGraphics->EndScene();
}
//------------------------------------------------------------------------------------------------------
void CApplication::Update()
{

}
//------------------------------------------------------------------------------------------------------
void CApplication::Render()
{

}
//------------------------------------------------------------------------------------------------------
void CApplication::Register(CApplication * pApp, const std::string &strName, int iWidth, int iHeight, bool bFullScreen)
{
	m_hApplication = pApp;
	m_windowRef.bIsFullScreen = bFullScreen;
	m_windowRef.strAppName = strName;
	m_windowRef.iWidth = iWidth;
	m_windowRef.iHeight = iHeight;
}
//------------------------------------------------------------------------------------------------------
CApplication * CApplication::GetInstance()
{
	return m_hApplication;
}
//------------------------------------------------------------------------------------------------------
void CApplication::Release()
{
	if(m_hApplication)
	{
		delete m_hApplication;
		m_hApplication = NULL;
	}
}