//IndexBuffer.cpp
#include "../dx_base/graphics.h"
#include "MyIndexBuffer.h"

#include <d3d9.h>
#include <d3dx9.h>


MyIndexBuffer::MyIndexBuffer()
{
    m_pIB        = NULL; 
    m_numIndices = 0; 
}
//----------------------------------------------------------------------------------------------
void MyIndexBuffer::Release()
{
    SAFE_RELEASE( m_pIB );
    m_numIndices = 0;
}
//----------------------------------------------------------------------------------------------
bool MyIndexBuffer::CreateBuffer( IDirect3DDevice9* pDevice, UINT numIndices, D3DFORMAT format, bool bDynamic )
{
    m_numIndices = numIndices;

    // Dynamic buffers can't be in D3DPOOL_MANAGED
    D3DPOOL pool = bDynamic ? D3DPOOL_DEFAULT : D3DPOOL_MANAGED;
    UINT size = (format == D3DFMT_INDEX32) ? sizeof( UINT ) : sizeof( USHORT );
    DWORD usage = bDynamic ? D3DUSAGE_WRITEONLY | D3DUSAGE_DYNAMIC : D3DUSAGE_WRITEONLY;

    if( FAILED( pDevice->CreateIndexBuffer( m_numIndices * size, usage, format, pool, &m_pIB, NULL ) ) )
    {
        DebugError( "CreateIndexBuffer failed." );
        return false;
    }
    return true;
}
//----------------------------------------------------------------------------------------------
bool MyIndexBuffer::SetData( UINT numIndices, void *pIndices, DWORD flags )
{
    if ( m_pIB == NULL )
        return false;

    char *pData;

    D3DINDEXBUFFER_DESC desc;
    m_pIB->GetDesc( &desc );
    UINT size = (desc.Format == D3DFMT_INDEX32) ? sizeof( UINT ) : sizeof( USHORT );

    // Lock the index buffer
    if ( FAILED( m_pIB->Lock( 0, 0, (void**)&pData, flags ) ) )
        return false;

    memcpy( pData, pIndices, numIndices * size );

    // Unlock index buffer
    if ( FAILED( m_pIB->Unlock() ) )
        return false;

    return true;
}