//MyVector2.h
#include <math.h>
#include "MyVector2.h"

MyVector2::MyVector2() : x(0.0f), y(0.0f)
{
}
//---------------------------------------------------------------------------------------------
MyVector2::MyVector2(float _x, float _y) : x(_x), y(_y)
{
}
//---------------------------------------------------------------------------------------------
const float& MyVector2::operator[] (int i) const
{
   return ((&x)[i]);
}
//---------------------------------------------------------------------------------------------
float& MyVector2::operator[] (int i)
{
   return ((&x)[i]);
}
//---------------------------------------------------------------------------------------------
// VECTOR = scalar * VECTOR
MyVector2 MyVector2::operator *(float num) const
{
	return MyVector2(x * num, y * num);
}
//---------------------------------------------------------------------------------------------
MyVector2 operator *(float num, const MyVector2& vec)
{
   return MyVector2(vec.x * num, vec.y * num);
}
//---------------------------------------------------------------------------------------------
// .
float MyVector2::operator *(const MyVector2& v) const
{
    return (x * v.x + y * v.y);
}
//---------------------------------------------------------------------------------------------
MyVector2& MyVector2::operator =(const MyVector2& v)
{
    x = v.x;
    y = v.y;
    return (*this);
}
//---------------------------------------------------------------------------------------------
MyVector2 MyVector2::operator +(const MyVector2& v) const
{
    return MyVector2(x + v.x, y + v.y);
}
//---------------------------------------------------------------------------------------------
MyVector2 MyVector2::operator -(const MyVector2& v) const
{
    return MyVector2(x - v.x, y - v.y);
}
//---------------------------------------------------------------------------------------------
MyVector2& MyVector2::operator +=(const MyVector2& v)
{
    x += v.x;
    y += v.y;
    return (*this);
}
//---------------------------------------------------------------------------------------------
MyVector2 MyVector2::operator /(float t) const
{
    float f = 1.0f / t;
    return (MyVector2(x * f, y * f));
}
//---------------------------------------------------------------------------------------------
MyVector2& MyVector2::operator /=(float t)
{
    float f = 1.0f / t;
    x *= f;
    y *= f;
    return (*this);
}
//---------------------------------------------------------------------------------------------
MyVector2& MyVector2::Normalize()
{
    return (*this /= sqrtf(x * x + y * y ));
}
//---------------------------------------------------------------------------------------------
float GetLineMagnitude(const MyVector2& src, const MyVector2& dest)
{
	MyVector2   vec;

	vec[0] = dest[0] - src[0];
	vec[1] = dest[1] - src[1];

	return (float)sqrt(vec * vec);
}
//---------------------------------------------------------------------------------------------
//u = ((P3  - P1) dot d)/(d dot d) = ((P3  - P1) dot d)/ (|| d||) 
//P3 Point̴
bool GetLengthLineAndPoint(const MyVector2& src, const MyVector2& dest, const MyVector2& point, float* fLength)
{
	float   u, lineLength;
	MyVector2  p;

	lineLength = GetLineMagnitude(src, dest);

	//Ʒ  u   Ȱ
	//u = (( (point.x - src.x) * (dest.x - src.x)) + ((point.y - src.y) * (dest.y - src.y))  )) / (lineLength * lineLength);
	//u = u = (point - src) * (dest - src) / ((dest - src) * (dest - src));
	u = (point - src) * (dest - src) / (lineLength * lineLength);
    
	//    ʴ 
	if(u < 0.0f || u > 1.0f)
		return false;

	//p.x = src.x + u * (dest.x - src.x);
	//p.y = src.y + u * (dest.y - src.y);
	// ּ   ´
	p = src + u*(dest - src);

	//  p point ũ⸦ Ѵ.
	*fLength = GetLineMagnitude(p, point);
	return true;
}
