//xfile.cpp
#include <d3d9.h>
#include <d3dx9.h>
#include <Dxerr.h>
#include <tchar.h >

#include "../dx_base/graphics.h"
#include "dxeffect.h"
#include "xfile.h"


CXFile::CXFile()
{
	m_pMesh = NULL;
	m_pDevice = NULL;
	m_pDXEffect = NULL;
}
//-------------------------------------------------------------------------------
CXFile::~CXFile()
{
	Release();
}
//-------------------------------------------------------------------------------
bool CXFile::Load(IDirect3DDevice9* pDevice, char* szFileName)
{
	HRESULT hr = 0;

	ID3DXBuffer* adjBuffer  = 0;
	ID3DXBuffer* matBuffer = 0;
	DWORD        numMtrls   = 0;

	hr = D3DXLoadMeshFromX(szFileName, D3DXMESH_MANAGED, pDevice, &adjBuffer, &matBuffer, 0, &numMtrls, &m_pMesh);
	if(FAILED(hr))
		return false;

	if( matBuffer != 0 && numMtrls != 0 )
	{
		D3DXMATERIAL* mat = (D3DXMATERIAL*)matBuffer->GetBufferPointer();

		for(int i = 0; i < (int)numMtrls; i++)
		{
			//MatD3D Ӽ ambient   Ƿ  ̸ Ѵ.
			mat[i].MatD3D.Ambient = mat[i].MatD3D.Diffuse;
			m_materialContainer.push_back( mat[i].MatD3D );

			if( mat[i].pTextureFilename != 0 )
			{
				IDirect3DTexture9* tex = 0;
				D3DXCreateTextureFromFileEx( pDevice, mat[i].pTextureFilename, D3DX_DEFAULT, D3DX_DEFAULT, 
                                       D3DX_DEFAULT, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED, D3DX_DEFAULT, 
									   D3DX_DEFAULT, 0, NULL, NULL, &tex);
				m_textureContainer.push_back( tex );
			}
			else
				m_textureContainer.push_back( 0 );
		}
	}

	SAFE_RELEASE(matBuffer); 

	// Optimize the mesh.
	hr = m_pMesh->OptimizeInplace(D3DXMESHOPT_ATTRSORT | D3DXMESHOPT_COMPACT | D3DXMESHOPT_VERTEXCACHE,
		(DWORD*)adjBuffer->GetBufferPointer(), 0, 0, 0);

	SAFE_RELEASE(adjBuffer); 

	if(FAILED(hr))
		return false;

	m_pDevice = pDevice;

	m_pDXEffect = new CDXEffect;
	bool bResult = m_pDXEffect->Load(pDevice, "test.fx");
	return true;
}
//-------------------------------------------------------------------------------
bool CXFile::SetAttribute(CDXEffect* pDXEffect, IDirect3DTexture9* pkTexture, D3DMATERIAL9* pkMaterial)
{
	ID3DXEffect* pEffect = m_pDXEffect->GetEffect();

	pEffect->SetVector("k_a", (D3DXVECTOR4 *)&pkMaterial->Ambient);
	pEffect->SetVector("k_d", (D3DXVECTOR4 *)&pkMaterial->Diffuse);
	pEffect->SetTexture("baseMap", pkTexture);
	return true;
}
//-------------------------------------------------------------------------------
void CXFile::Draw()
{
	HRESULT hr;
	ID3DXEffect* pEffect = m_pDXEffect->GetEffect();

	D3DXMATRIX   matWorld, matView, matProj;
	m_pDevice->GetTransform(D3DTS_WORLD,&matWorld);
	m_pDevice->GetTransform(D3DTS_VIEW, &matView);
	m_pDevice->GetTransform(D3DTS_PROJECTION, &matProj);

	D3DXMATRIX matWorldView = matWorld * matView;
    D3DXMATRIX matWorldViewProj = matWorld * matView * matProj;

	_V(pEffect->SetMatrix("g_matWorldView", &matWorldView));
	_V(pEffect->SetMatrix("g_matWorldViewProj", &matWorldViewProj));

	DWORD cPasses;
	for(int i = 0; i < (int)m_materialContainer.size(); ++i)
	{  
		bool bResult;
		if(m_textureContainer[i])
			bResult = m_pDXEffect->SetTechnique("BasicTest");
		else
			bResult = m_pDXEffect->SetTechnique("NoTextureTest");

		cPasses = m_pDXEffect->Begin();
		for(UINT uPassCount = 0; uPassCount < cPasses; ++uPassCount)
		{
			bResult = m_pDXEffect->BeginPass(uPassCount);
			SetAttribute(m_pDXEffect, m_textureContainer[i], &m_materialContainer[i]);
			bResult = m_pDXEffect->CommitChanges();
			m_pMesh->DrawSubset(i);
			m_pDXEffect->EndPass();
		}

		m_pDXEffect->End();
	}	
}
//-------------------------------------------------------------------------------
void CXFile::Release()
{
	if(m_pDevice)
	{
		SAFE_DELETE(m_pDXEffect);
		SAFE_RELEASE(m_pMesh);

		for(int i = 0; i < (int)m_textureContainer.size(); i++)
			SAFE_RELEASE( m_textureContainer[i] );

		m_materialContainer.clear();
		m_textureContainer.clear();
		m_pDevice = NULL;
	}
}
