//testapp.cpp
#include <windows.h>
#include "./dx_base/graphics.h"
#include "testapp.h"
#include "./dx_util/xfile.h"
#include "./dx_util/dxeffect.h"
#include "./dx_util/MyRenderer.h"
#include "./dx_util/MyMesh.h"
#include "./dx_util/MyTexture.h"
#include "./dx_util/MyMaterial.h"

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	CTestApp * pApp = new CTestApp;

	pApp->m_mainParam.hInstance = hInstance;
	pApp->m_mainParam.lpCmdLine = lpCmdLine;
	pApp->m_mainParam.nCmdShow = nCmdShow;
	
	pApp->Register(pApp, "Z-UP ǥ ٲٱ", 640, 480); 
	pApp->Run();
	pApp->Release();
	return 0;
}
//---------------------------------------------------------------------------------
CTestApp::CTestApp()
{
	m_pXFile = NULL;
	m_pEffect = NULL;
	m_cameraRef.wheel = -5.0f;
}
//---------------------------------------------------------------------------------
CTestApp::~CTestApp()
{

}
//---------------------------------------------------------------------------------
void CTestApp::InitDirectionalLight()
{
	IDirect3DDevice9 * pDevice = m_pGraphics->GetDevice();

	D3DLIGHT9 light;
	ZeroMemory( &light, sizeof(light) );
	light.Type = D3DLIGHT_DIRECTIONAL;

	D3DXVECTOR3 vecDir;
	vecDir = D3DXVECTOR3(0.5f, 0.5f, -0.1f);
	D3DXVec3Normalize((D3DXVECTOR3*)&light.Direction, &vecDir );

	// set directional light diffuse color
	light.Type  = D3DLIGHT_DIRECTIONAL;
	light.Diffuse = D3DXCOLOR( 1.f, 1.f, 1.f, 1.f );
	light.Ambient = D3DXCOLOR( 0.f, 0.f, 0.f, 1.f );

	pDevice->SetLight( 0, &light );
	pDevice->LightEnable( 0, TRUE );
	pDevice->SetRenderState( D3DRS_LIGHTING, TRUE );
	pDevice->SetRenderState( D3DRS_ZENABLE,  TRUE );
	//Ambient Light 
	pDevice->SetRenderState(D3DRS_AMBIENT, 0x00202020);
}
//---------------------------------------------------------------------------------
bool CTestApp::PostGraphics()
{
	CApplication::PostGraphics();
	InitDirectionalLight();

	IDirect3DDevice9 * pDevice = m_pGraphics->GetDevice();
	
	m_pXFile = new CXFile;
	m_pXFile->Load(pDevice, "airplane 2.x");

	m_pRenderer = new MyRenderer;
	m_pMesh = new MyMesh;
	m_pTexture = new MyTexture("kim.jpg");
	m_pMaterial = new MyMaterial();
	m_pMaterial->SetEmissive(0.0f, 0.0f, 0.0f);

	//Vertex
    MyVector3 p1 = MyVector3(-1.0f,-1.0f, 1.0f); 
    MyVector3 p2 = MyVector3( 1.0f,-1.0f, 1.0f); 
    MyVector3 p3 = MyVector3( 1.0f,-1.0f,-1.0f); 
    MyVector3 p4 = MyVector3(-1.0f,-1.0f,-1.0f); 
    MyVector3 p5 = MyVector3(-1.0f, 1.0f, 1.0f); 
    MyVector3 p6 = MyVector3(-1.0f, 1.0f,-1.0f); 
    MyVector3 p7 = MyVector3( 1.0f, 1.0f,-1.0f); 
    MyVector3 p8 = MyVector3( 1.0f, 1.0f, 1.0f);

	MyVector3* pVertex = new MyVector3[24];
	//front
	pVertex[0] = p1;	pVertex[1] = p2; 
	pVertex[2] = p3;	pVertex[3] = p4;
	//back
	pVertex[4] = p5;	pVertex[5] = p6; 
	pVertex[6] = p7;	pVertex[7] = p8;
	//top
	pVertex[8] = p5;	pVertex[9] = p8; 
	pVertex[10] = p2;	pVertex[11] = p1;
    //bottom
	pVertex[12] = p6;	pVertex[13] = p4; 
	pVertex[14] = p3;	pVertex[15] = p7;
    //right
	pVertex[16] = p2;	pVertex[17] = p8; 
	pVertex[18] = p7;	pVertex[19] = p3;
	//left
	pVertex[20] = p1;	pVertex[21] = p4; 
	pVertex[22] = p6;	pVertex[23] = p5;

	//normal
	MyVector3* pNormal = new MyVector3[24];
	//front
	pNormal[0] = MyVector3(0.0f, -1.0f, 0.0f);
	pNormal[1] = pNormal[2] = pNormal[3] = pNormal[0];
	//back
	pNormal[4] = MyVector3(0.0f, 1.0f, 0.0f);
	pNormal[5] = pNormal[6] = pNormal[7] = pNormal[4];
	//top
	pNormal[8] = MyVector3(0.0f, 0.0f, 1.0f);
	pNormal[9] = pNormal[10] = pNormal[11] = pNormal[8];
    //bottom
	pNormal[12] = MyVector3(0.0f, 0.0f, -1.0f);
	pNormal[13] = pNormal[14] = pNormal[15] = pNormal[12];
    //right
	pNormal[16] = MyVector3(1.0f, 0.0f, 0.0f);
	pNormal[17] = pNormal[18] = pNormal[19] = pNormal[16];
	//left
	pNormal[20] = MyVector3(-1.0f, 0.0f, 0.0f);
	pNormal[21] = pNormal[22] = pNormal[23] = pNormal[20];

	//ؽ
	MyVector2 t1 = MyVector2( 0.0f,  0.0f);
	MyVector2 t2 = MyVector2( 1.0f,  0.0f);
	MyVector2 t3 = MyVector2( 1.0f,  1.0f);
	MyVector2 t4 = MyVector2( 0.0f,  1.0f);

	MyVector2* pUV = new MyVector2[24];
	//front
    pUV[0] = t1;	pUV[1] = t2;	
	pUV[2] = t3;	pUV[3] = t4;
    //back
    pUV[4] = t2;	pUV[5] = t3;	
	pUV[6] = t4;	pUV[7] = t1;
	//top
    pUV[8] = t1;	pUV[9] = t2;	
	pUV[10] = t3;	pUV[11] = t4;
	//bottom
    pUV[12] = t4;	pUV[13] = t1;	
	pUV[14] = t2;	pUV[15] = t3;
	//right
    pUV[16] = t1;	pUV[17] = t2;	
	pUV[18] = t3;	pUV[19] = t4;
	//left
	pUV[20] = t2;	pUV[21] = t3;	
	pUV[22] = t4;	pUV[23] = t1;

	m_pMesh->SetNumber( 24, 12);
	m_pMesh->SetVertexData( pVertex);
	m_pMesh->SetVertexNormal( pNormal);
	m_pMesh->SetTextureUV( pUV); 

	//ε 
	USHORT* pIndex = new USHORT[36];
	pIndex[0] = 2;	 pIndex[1] = 1;		pIndex[2] = 0;
	pIndex[3] = 0;	 pIndex[4] = 3;		pIndex[5] = 2;
	pIndex[6] = 6;	 pIndex[7] = 5;		pIndex[8] = 4;
	pIndex[9] = 4;	 pIndex[10] = 7;	pIndex[11] = 6;
	pIndex[12] = 10; pIndex[13] = 9;	pIndex[14] = 8;
	pIndex[15] =  8; pIndex[16] = 11;	pIndex[17] = 10;
	pIndex[18] = 14; pIndex[19] = 13;	pIndex[20] = 12;
	pIndex[21] = 12; pIndex[22] = 15;	pIndex[23] = 14;
	pIndex[24] = 18; pIndex[25] = 17;	pIndex[26] = 16;
	pIndex[27] = 16; pIndex[28] = 19;	pIndex[29] = 18;
	pIndex[30] = 22; pIndex[31] = 21;	pIndex[32] = 20;
	pIndex[33] = 20; pIndex[34] = 23;	pIndex[35] = 22;
	m_pMesh->SetVertexIndex(36, pIndex);
	return true;
}
//---------------------------------------------------------------------------------
void CTestApp::Cleanup()
{
	SAFE_DELETE(m_pEffect);
	SAFE_DELETE(m_pXFile);
	SAFE_DELETE(m_pTexture);
	SAFE_DELETE(m_pMaterial);
	SAFE_DELETE(m_pMesh);
	SAFE_DELETE(m_pRenderer);
	CApplication::Cleanup();
}
//---------------------------------------------------------------------------------
void CTestApp::Update()
{

}
//---------------------------------------------------------------------------------
void CTestApp::Render()
{
	D3DXMATRIX   matTranslation, matRot, matWorld;
	
	D3DXMatrixTranslation( &matTranslation, 0.0f, m_cameraRef.wheel * 2.0f, 0.0f);
	D3DXMatrixRotationYawPitchRoll( &matRot, 0.0f, D3DXToRadian(-m_cameraRef.fRotateY), D3DXToRadian(-m_cameraRef.fRotateX));
	IDirect3DDevice9 * pDevice = m_pGraphics->GetDevice();

	m_pTexture->SetTexture( pDevice);
	m_pMaterial->SetMaterial(pDevice);

	matWorld = matRot * matTranslation;
	pDevice->SetTransform( D3DTS_WORLD, &matWorld);
	m_pRenderer->Render(pDevice, *m_pMesh);
	//m_pXFile->Draw();
}
//---------------------------------------------------------------------------------
bool CTestApp::ProcessKey(HWND hWnd, WPARAM wKey)
{
	return CApplication::ProcessKey(hWnd, wKey);
}


